#include "framecpp/config.h"

extern "C" {
#include <zlib.h>
} // extern "C"

#include <stdexcept>

#include "general/types.hh"
#include "general/autoarray.hh"

#include "framecpp/Common/CompressionGZip.hh"

namespace FrameCPP
{
  namespace Compression
  {
    //-------------------------------------------------------------------
    /// The routines defined in this namespace are specific to the
    /// GZip compression algorithm.
    /// Routines are defined for both compression and decompression
    /// of data buffers.
    //-------------------------------------------------------------------
    namespace GZip
    {
      //-----------------------------------------------------------------
      /// The data must be in an uncompressed state before this is
      /// called.
      //-----------------------------------------------------------------
      void
      Compress( const CHAR_U* DataIn, INT_8U NBytesIn, size_t Level,
		General::AutoArray< CHAR_U >& DataOut, INT_8U& NBytesOut )
      {
	if ( Level == 0 )
	{
	  return;
	}
	if ( Level > 9 )
	{
	  Level = 9;
	}
    
	uLongf bufferSize = uLongf( NBytesIn * 1.001 + 13 );
    
	General::AutoArray< CHAR_U > buffer( new CHAR_U[ bufferSize ] );
#if HAVE_LIBZ_COMPRESS2
	::compress2( buffer.get( ), &bufferSize, DataIn, NBytesIn, Level );
#else
	::compress( buffer.get( ), &bufferSize, DataIn, NBytesIn );
#endif

	NBytesOut = bufferSize;
	DataOut.reset( new CHAR_U[ NBytesOut ] );
	std::copy( &buffer[ 0 ], &( buffer[ NBytesOut] ), &(DataOut[ 0 ]) );
      } // func - Compress

      //-----------------------------------------------------------------
      /// This will uncompress the data using the appropriate GZip
      /// call.
      //-----------------------------------------------------------------
      void
      Expand( const CHAR_U* DataIn, INT_8U NBytesIn,
	      General::AutoArray< CHAR_U >& DataOut, INT_8U& NBytesOut )
      {
	//---------------------------------------------------------------
	// This may throw std::bad_alloc
	//---------------------------------------------------------------
	DataOut.reset( new CHAR_U[ NBytesOut ] );
	uLongf uncompressedLength( NBytesOut );

	int error = uncompress( DataOut.get( ),
				&uncompressedLength,
				DataIn,
				NBytesIn );

	//---------------------------------------------------------------
	// Check the return status from the uncompress
	//---------------------------------------------------------------
	if ( error != Z_OK )
	{
	  DataOut.reset( (CHAR_U*)NULL );
	  switch( error )
	  {
	  case Z_MEM_ERROR:
	    throw std::bad_alloc();
	    break;
	
	  case Z_BUF_ERROR:
	  case Z_DATA_ERROR:
	    throw std::runtime_error( "Compressed data is corrupt." );
	    break;
                    
	  default:
	    throw std::runtime_error( "An unknown error occurred during "
				      "uncompression." );
	  }
	}

	//---------------------------------------------------------------
	// Make sure we uncompressed the correct number of bytes.
	//---------------------------------------------------------------
	if ( uncompressedLength != NBytesOut )
	{
	  throw std::range_error( "Compressed data is corrupt" );
	}

	//---------------------------------------------------------------
	// Successfully expanded the buffer, so return the results
	//---------------------------------------------------------------
      } // func - Exapnd

    } // namespace - GZip

  } // namespace - Compression

} // namespace - FrameCPP
